/* -*- C++ -*-
 * Copyright ©2004 Hugo Mills <hugo@carfax.org.uk>
 *
 * This software is distributed under the terms of the GNU GPL v2
 * For more information on the GPL, see the file COPYING or 
 * visit http://www.gnu.org/
 *
 * This software is distributed without warranty
 */

#include "magellan/configuration.h"
#include "magellan/options.h"

class DeleterMap { 
public: 
	DeleterMap() { }
	template <class T> void operator()(T ob) { ob.second->destroy(); }
};

class DeleterSet { 
public:
	DeleterSet() { }
	template <class T> void operator()(T ob) { ob->destroy(); }
};

Configuration::~Configuration()
{
	_output->destroy();
	for_each(orbits.begin(), orbits.end(), DeleterMap());
	for_each(content.begin(), content.end(), DeleterMap());
	for_each(maps.begin(), maps.end(), DeleterMap());
	for_each(viewports.begin(), viewports.end(), DeleterMap());
	for_each(displays.begin(), displays.end(), DeleterSet());
}

void Configuration::update(void)
{
	for(std::map<std::string, XfOrbit*>::iterator it = orbits.begin();
		it != orbits.end();
		++it)
	{
		it->second->reset();
	}
}

void Configuration::render(void)
{
	if(opts->debug_render() >= 1)
		std::cerr << "Rendering started" << std::endl;

	for(std::set<MDisplay*>::iterator disp = displays.begin();
		disp != displays.end();
		disp++)
	{
		if(opts->debug_render() >= 2)
			std::cerr << "  Rendering from " << *disp << std::endl;
		(*disp)->plot(_output);
	}

	if(opts->debug_render() >= 1)
		std::cerr << "Rendering finished" << std::endl;
}

void Configuration::output(void)
{
	_output->commit();
}

/* Methods to get the different types of plugin */
XfView* Configuration::getviewport(const std::string& name) const
{
	std::map<std::string, XfView*>::const_iterator rv = 
		viewports.find(name);
	if(rv == viewports.end())
		return NULL;
	return rv->second;
}

XfMap* Configuration::getprojection(const std::string& name) const
{
	std::map<std::string, XfMap*>::const_iterator rv = 
		maps.find(name);
	if(rv == maps.end())
		return NULL;
	return rv->second;
}

XfOrbit* Configuration::getorbit(const std::string& name) const
{
	std::map<std::string, XfOrbit*>::const_iterator rv = 
		orbits.find(name);
	if(rv == orbits.end())
		return NULL;
	return rv->second;
}

Content* Configuration::getcontent(const std::string& name) const
{
	std::map<std::string, Content*>::const_iterator rv = 
		content.find(name);
	if(rv == content.end())
	{
		std::cerr << "Content object " << name << " not found" << std::endl;
		return NULL;
	}

	return rv->second;
}

/* Template specialisations for the different types of plugin */
template <>
XfOrbit* Configuration::getplugin<XfOrbit>(const std::string& s) const
{ return getorbit(s); }

template <>
XfMap* Configuration::getplugin<XfMap>(const std::string& s) const
{ return getprojection(s); }

template <>
XfView* Configuration::getplugin<XfView>(const std::string& s) const
{ return getviewport(s); }

template <>
Content* Configuration::getplugin<Content>(const std::string& s) const
{ return getcontent(s); }
