/* -*- C++ -*-
 * Copyright ©2004 Hugo Mills <hugo@carfax.org.uk>
 *
 * This software is distributed under the terms of the GNU GPL v2
 * For more information on the GPL, see the file COPYING or 
 * visit http://www.gnu.org/
 *
 * This software is distributed without warranty
 */

#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include <list>
#include <set>
#include <map>

#include "magellan/output.h"
#include "magellan/xforbit.h"
#include "magellan/content.h"
#include "magellan/xfmap.h"
#include "magellan/xfview.h"
#include "magellan/mdisplay.h"

#include "magellan/configlexer.h"

// Main class
class Configuration
{
public:
	Configuration(ConfigLexer*);
	virtual ~Configuration();

	void update(void);  // Update orbital positions
	void render(void);  // Render to the target bitmap
	void output(void);  // Output the target bitmap

	bool repeated(void) const { return _output->repeated(); }
	time_t updatetime(void) const { return _output->updatetime(); }

	// Get plugins of type known at compile time
	XfOrbit* getorbit(const std::string&) const;
	XfMap* getprojection(const std::string&) const;
	XfView* getviewport(const std::string&) const;
	Content* getcontent(const std::string&) const;

	// Get plugins of arbitrary type
	template<class T>
	T* getplugin(const std::string&) const;

private:
	Output* _output; // The one output device
	std::map<std::string, XfOrbit*> orbits;
	std::map<std::string, Content*> content;
	std::map<std::string, XfMap*> maps;
	std::map<std::string, XfView*> viewports;
	std::set<MDisplay*> displays;
};

#endif
