/* -*- C++ -*-
 * Copyright ©2004 Hugo Mills <hugo@carfax.org.uk>
 *
 * This software is distributed under the terms of the GNU GPL v2
 * For more information on the GPL, see the file COPYING or 
 * visit http://www.gnu.org/
 *
 * This software is distributed without warranty
 */

#ifndef VALUES_H
#define VALUES_H

#include <limits>
#include <cmath>
#include <cstdlib>
#include <string>

#undef max

template <class T>
inline T defvalue() { return T(); }

template <>
inline double defvalue()
{ return std::numeric_limits<double>::max(); }
template <>
inline long defvalue()
{ return std::numeric_limits<long>::max(); }
template <>
inline int defvalue()
{ return std::numeric_limits<int>::max(); }

// Value conversion -- could be done with strstreams to avoid the
// specialisations
template <class T>
inline void convert(T* var, const std::string& val) { *var = val; }

template <>
inline void convert(int* var, const std::string& val)
{ *var = atoi(val.c_str()); }
template <>
inline void convert(long* var, const std::string& val)
{ *var = atol(val.c_str()); }
template <>
inline void convert(double* var, const std::string& val)
{ *var = atof(val.c_str()); }

// Set a value if currently unset
template <class T>
void setdef(T& var, const T& def)
{ if(var == defvalue<T>()) var = def; }

// Error checking functions
template <class T>
void required(const T& var)
{
	if(var == defvalue<T>())
	{
		// Syntax error -- required value not set
		std::cerr << "Syntax error -- required value not set" << std::endl;
		exit(1);
	}
}

template <class T>
void invalid(const T& var)
{
	if(var != defvalue<T>())
	{
		// Syntax error -- value set when not valid
		std::cerr << "Value cannot be specified here" << std::endl;
		exit(1);
	}
}

// Units conversion
const double DEG_TO_RAD = M_PI/180.0;
const double RAD_TO_DEG = 180.0/M_PI;

inline double degrad(double deg) { return deg*DEG_TO_RAD; }
inline double raddeg(double rad) { return rad*RAD_TO_DEG; }

#endif
