/* -*- C++ -*-
 * Copyright ©2005 Hugo Mills <hugo@carfax.org.uk>
 *
 * This software is distributed under the terms of the GNU GPL v2
 * For more information on the GPL, see the file COPYING or 
 * visit http://www.gnu.org/
 *
 * This software is distributed without warranty
 */

#include "magellan/content.h"

#include <vector>
#include <queue>
#include <fstream>

#include "magellan/options.h"
#include "magellan/vectors.h"
#include "content_outline_rawcontourset.h"

class Content_Outline : public Content
{
public:
	Content_Outline(
		const std::string&
		);
	virtual ~Content_Outline();

	virtual void plot(Output*, XfView*, XfMap*, XfOrbit*, XfSphere*);

	static Content* parse(
		ConfigLexer* lex,
		const std::string& type,
		const std::string& subtype = ""
		);

private:
};

extern "C" PluginInfo* getmodules(int i)
{
	if(i==0)
	{
		PluginInfo_Content* pi = new PluginInfo_Content();

		pi->name = "Outline content";
		pi->author = "Hugo Mills";
		pi->copyright = "©2005 Hugo Mills";
		pi->version = 0x00000;
		pi->parser = Content_Outline::parse;
		return pi;
	}
	else
		return NULL;
}

Content* Content_Outline::parse(
	ConfigLexer* lex,
	const std::string& type,
	const std::string& subtype
	)
{
	if(type != "outline")
		return NULL;

	std::string filename = "";

	while(!lex->eos()) {
		if(readkeyedvalue(lex, "file", &filename)
			)
		{
		}
		else
			throw(UnknownConfigToken(lex));
	}

	if(filename == "")
		throw PluginInitError("Filename not specified");
	
	std::string tmp = opts->find_data_file("outline/", filename);
	if(tmp == "")
		throw(PluginInitError("File '" + filename + "' not found"));

	return new Content_Outline(filename);
}

Content_Outline::Content_Outline(
	const std::string& filename
	)
{
	// Load data from file
	std::istream in = new std::ifstream(filename);
	while(!in.eof())
	{
		// Stuff
	}
}

Content_Outline::~Content_Outline()
{
}

