/* -*- C++ -*-
 * Copyright ©2004 Hugo Mills <hugo@carfax.org.uk>
 *
 * This software is distributed under the terms of the GNU GPL v2
 * For more information on the GPL, see the file COPYING or 
 * visit http://www.gnu.org/
 *
 * This software is distributed without warranty
 */

#include "xfmap_cylorthographic.h"

extern "C" PluginInfo* getmodules(int i)
{
	if(i==0)
	{
		PluginInfo_Map* pi = new PluginInfo_Map();

		pi->name = "Cylindrical Orthographic";
		pi->author = "Hugo Mills";
		pi->copyright = "©2005 Hugo Mills";
		pi->version = 0x10000;
		pi->parser = XfMap_CylindricalOrthographic::parser;
		return pi;
	}
	else
		return NULL;
}

XfMap* XfMap_CylindricalOrthographic::parser(
	ConfigLexer* lex,
	const std::string& type,
	const std::string& subtype
	)
{
	if(type == "cylindrical" && subtype == "orthographic")
	{
		double parallel = 0.0;

		while(!lex->eos()) {
			if(!readkeyedvalue(lex, "standard-parallels", &parallel))
				throw(UnknownConfigToken(lex));
		}

		return new XfMap_CylindricalOrthographic(degrad(parallel));
	}
	else
		return NULL;
}

bool XfMap_CylindricalOrthographic::f(dvec2& result, const dvec3& pos) const
{
	result[0] = atan2(pos[0], pos[2]) / M_PI;
	result[1] = pos[1] / scale / M_PI;

	return true;
}

bool XfMap_CylindricalOrthographic::g(dvec3& result, const dvec2& pos) const
{
	result[1] = pos[1] * scale * M_PI;

	if(result[1] < -1 || result[1] > 1)
		return false;
	if(pos[0] < -1 || pos[0] > 1)
		return false;

	double m = sqrt(1-result[1]*result[1]);
	double a = pos[0] * M_PI;

	result[0] = m * sin(a);
	result[2] = m * cos(a);

	return true;
}
